@@ -13,7 +13,7 @@ use URI;
 use URI::db;
 use URI::QueryParam;
 
-our $VERSION = '0.016';
+our $VERSION = '0.017';
 
 has dsn => sub { _url_from_file(shift->_tempfile)->dbi_dsn };
 has max_connections => 5;
@@ -48,21 +48,22 @@ sub db {
   return Mojo::SQLite::Database->new(dbh => $self->_dequeue, sqlite => $self);
 }
 
-sub from_filename { shift->from_string(_url_from_file(shift)) }
+sub from_filename { shift->from_string(_url_from_file(shift, shift)) }
 
 sub from_string {
-  my ($self, $str, $os) = @_;
+  my ($self, $str) = @_;
   return $self unless $str;
   my $url = URI->new($str);
 
   # Options
   my $options = $url->query_form_hash;
+  $url->query_param_delete($_) for $url->query_param;
   @{$self->options}{keys %$options} = values %$options;
 
   # Parse URL based on scheme
   $url->scheme('file') unless $url->has_recognized_scheme;
   if ($url->scheme eq 'file') {
-    $url = _url_from_file(defined $os ? $url->file($os) : $url->file);
+    $url = _url_from_file($url->file);
   } elsif ($url->scheme ne 'db') {
     $url = URI::db->new($url);
   }
@@ -73,7 +74,7 @@ sub from_string {
   
   # Temp database file
   $url->dbname($self->_tempfile) if $url->dbname eq ':temp:';
-
+  
   return $self->dsn($url->dbi_dsn);
 }
 
@@ -98,7 +99,13 @@ sub _enqueue {
 
 sub _tempfile { catfile(shift->{tempdir} = File::Temp->newdir, 'sqlite.db') }
 
-sub _url_from_file { URI::db->new->Mojo::Base::tap(engine => 'sqlite')->Mojo::Base::tap(dbname => shift) }
+sub _url_from_file {
+  my $url = URI::db->new;
+  $url->engine('sqlite');
+  $url->dbname(shift);
+  if (my $options = shift) { $url->query_form_hash($options) }
+  return $url;
+}
 
 1;
 
@@ -325,11 +332,11 @@ gracefully by holding on to it only for short amounts of time.
 
 =head2 from_filename
 
-  $sql = $sql->from_filename('C:\\Documents and Settings\\foo & bar.db');
+  $sql = $sql->from_filename('C:\\Documents and Settings\\foo & bar.db', $options);
 
 Parse database filename directly. Unlike L</"from_string">, the filename is
-parsed as a local filename and not a URL, and L</"options"> must be specified
-separately.
+parsed as a local filename and not a URL. A hashref of L</"options"> may be
+passed as the second argument.
 
   # Absolute filename
   $sql->from_filename('/home/fred/data.db');
@@ -344,7 +351,7 @@ separately.
   my $db = $sql->from_filename(':memory:')->db;
 
   # Additional options
-  $sql->from_filename($filename)->tap(sub { $_->options->{PrintError} = 1 });
+  $sql->from_filename($filename, { PrintError => 1 });
 
 =head2 from_string
 
@@ -357,9 +364,10 @@ Parse configuration from connection string. Connection strings are parsed as
 URLs, so you should construct them using a module like L<Mojo::URL>,
 L<URI::file>, or L<URI::db>. For portability on non-Unix-like systems, either
 construct the URL with the C<sqlite> scheme, or use L<URI::file/"new"> to
-construct a URL with the C<file> scheme. A URL with no scheme will be parsed
-as a C<file> URL. If specified, the hostname must be C<localhost>. If the URL
-has a query string, it will be parsed and applied to L</"options">.
+construct a URL with the C<file> scheme. A URL with no scheme will be parsed as
+a C<file> URL, and C<file> URLs are parsed according to the current operating
+system. If specified, the hostname must be C<localhost>. If the URL has a query
+string, it will be parsed and applied to L</"options">.
 
   # Absolute filename
   $sql->from_string('sqlite:////home/fred/data.db');
@@ -380,7 +388,7 @@ has a query string, it will be parsed and applied to L</"options">.
   # Connection string must be a valid URL
   $sql->from_string(Mojo::URL->new->scheme('sqlite')->path($filename));
   $sql->from_string(URI::db->new->Mojo::Base::tap(engine => 'sqlite')->Mojo::Base::tap(dbname => $filename));
-  $sql->from_string(URI::file->new($filename, 'win32'));
+  $sql->from_string(URI::file->new($filename));
 
   # Temporary file database (default)
   $sql->from_string(':temp:');
